---
meta:
    description:
        A complete, fully operational, AWS Batch computing system

    parameters:
        has_priority_queue:
            description: Should the environment have a high priority queue?
            value: no

        ip_whitelist:
            description:
                The whitelisted IPs that will have access the ECS instances
            value: []

        sg_whitelist:
            description:
                The list of security groups that will have access to the instances
            value: []

        bid_percentage:
            description:
                The minimum percentage that a Spot Instance price must be when
                compared with the On-Demand price for that instance type
                before instances are launched.
            value:
                100

        ami_id:
            description:
                The AMI ID
            value:

        key_name:
            description:
                The name of the SSH key to inject in the EC2 instances
            value:

        max_cpus:
            description: Max. number of vCPUs
            value: 100

        min_cpus:
            description: Min. number of vCPUs
            value: 0

        state:
            description: Either ENABLED (default) or DISABLED
            value: ENABLED

        type:
            description: Either EC2 or SPOT
            value: SPOT

        bid_percentage:
            description: The Bid percentage for spot instances
            value: 100

        security_group_ids:
            description: A list of security group IDs, required if vpc_id is not provided
            value: []

        vpc_id:
            description: The VPC ID, required if security_group_ids is not provided
            value:

        subnet_ids:
            description: List of subnets where the instances will be launched
            value:

        tags:
            description: A dict of tags to associate to the EC2 instances
            value:
                Name: "{{__context.environment.name}}-{{__context.layer.name}}-{{__context.stage.lower()}}"

        instance_types:
            description: A list of instance types
            value:
                - c4.large
                - c4.xlarge
                - c4.2xlarge
                - c4.4xlarge
                - c4.8xlarge

        ec2_key_pair:
            description: The name of a SSH key to inject in the instances
            value:
