---
outputs:
    ComputeEnvironmentArn:
        Description:
            The ARN of the compute environment
        Value: {Ref: ComputeEnvironment}

    {% if has_priority_queue %}
    PriorityJobQueueArn:
        Description:
            The ARN of the priority job queue
        Value: {Ref: PriorityJobQueue}
    {% endif %}

    DefaultJobQueueArn:
        Description:
            The ARN of the default job queue
        Value: {Ref: DefaultJobQueue}
